// #autoload
// #name = SonicMark
// #category = Sonic's Scripts
// #version = 1.0
// #date = January 3, 2004
// #status beta
// #description = Waypoint players based on attributes.  Has both offense and defense oriented waypointing.  If a player is grabbing a lot and depending on the team a waypoint is placed on the player so that he/she/it is easier to track.
// #warrior = sonic9k
// #author = sonic9k
// #email = sonic9k@jobielan.com
// #config = SonicMarkOptions
// #include = support/callback.cs
// #include = support/loadout.cs
// #include = support/player_support.cs
// #include = support/stat_support.cs
// #include = support/kill_callbacks.cs
// #include = support/team_tracker.cs
// #include = support/flag_tracker.cs
// #readme = scripts/autoexec/sonicmark.txt
// #credit = ilys for idea from capperwaypoints.cs
//
//
////////////////////////////////////////////////////////////////////////////////
//
//  ToDos:
//
//  - Add weapon detection to kill callback.  Might be nice to boost
//    a sniper that never gets headshots.  Make sure to check that the
//    sniper shot was not a headshot which would be counted differently.
//
//  Problems:
//
//  - When switching teams even with callbacks it thought the player was on a
//    different team and had a friendly waypoint on player with Attack Capper
//    instead of Escort Capper.
//  - Internal testing waypointed 1 more killer than it should have.  Have to
//    some real world testing to confirm.
//
////////////////////////////////////////////////////////////////////////////////

// Read in prefs if there is one else create it.
if (isFile ("prefs/SonicMarkPrefs.cs"))
	exec ("prefs/SonicMarkPrefs.cs");
else
{
	$SonicMark::Enabled = true;

	$SonicMark::Enemy["capper", "enabled"] = true;
	$SonicMark::Enemy["capper", "max"] = 2;
	$SonicMark::Enemy["capper", "label"] = "Attack Capper";

	$SonicMark::Enemy["sniper", "enabled"] = false;
	$SonicMark::Enemy["sniper", "max"] = 2;
	$SonicMark::Enemy["sniper", "label"] = "Attack Sniper";

	$SonicMark::Enemy["shocklancer", "enabled"] = false;
	$SonicMark::Enemy["shocklancer", "max"] = 2;
	$SonicMark::Enemy["shocklancer", "label"] = "Attack Shocklancer";

	$SonicMark::Enemy["killer", "enabled"] = true;
	$SonicMark::Enemy["killer", "max"] = 2;
	$SonicMark::Enemy["killer", "label"] = "Attack Killer";

	$SonicMark::Enemy["fckiller", "enabled"] = false;
	$SonicMark::Enemy["fckiller", "max"] = 2;
	$SonicMark::Enemy["fckiller", "label"] = "Attack FC Killer";

	$SonicMark::Friendly["capper", "enabled"] = true;
	$SonicMark::Friendly["capper", "max"] = 2;
	$SonicMark::Friendly["capper", "label"] = "Escort Capper";

	$SonicMark::Friendly["teamkiller", "enabled"] = true;
	$SonicMark::Friendly["teamkiller", "max"] = 2;
	$SonicMark::Friendly["teamkiller", "label"] = "Avoid TKer";

	$SonicMark::EnemyOrder = "capper killer";
	$SonicMark::FriendlyOrder = "capper teamkiller";

	export ("$SonicMark::*", "prefs/SonicMarkPrefs.cs", false);
}

////////////////////////////////////////////////////////////////////////

// Create SonicMark object
if (!isObject (SonicMark))
{
	new ScriptObject (SonicMark)
	{
		class = SonicMark;
		version = "1.0";
		myTeam = 0;
		enemyTeam = 0;
		Player = "";
		PlayerCount = 0;
		KeybindsAdded = false;
		WaypointMagic = 7612872;
		UpdateInterval = 2000;
	};

	error ("Loading version "@SonicMark.version@" of SonicMark ...");
}


////////////////////////////////////////////////////////////////////////
// Global Functions
////////////////////////////////////////////////////////////////////////

// Add a named waypoint to the clientId
function AddWaypointToClient (%name, %clientId)
{
	%id = PlayerList.GetTargetIDbyID (%clientId);
	%target = createClientTarget (%id, "0 0 9999");
	%target.createWaypoint (%name);
	%target.myMagic = SonicMark.WaypointMagic;
	%target.myTargetId = %target;
	$ClientWaypoints.add (%target);

	//error ("SM: Adding waypoint \""@%name@"\" to player \""@PlayerList.getNameByID (%clientId)@"\", target ID = "@%target@", count = "@$ClientWaypoints.getCount());
}

// Clear waypoints by matching on magic number
function ClearWaypointsByMagic()
{
	%cnt = $ClientWaypoints.getCount();

	// First build a list of things to delete then delete them all at once
	for (%i = 0; %i < %cnt; %i++)
	{
		%o = $ClientWaypoints.getObject(%i);

		if (%o.myMagic == SonicMark.WaypointMagic)
			%obj = (%obj $= "") ? %o : %obj SPC %o;
	}

	for (%i=0; %i < getWordCount (%obj); %i++)
	{
		//error ("SM: Deleting waypoint "@%i@", target ID "@%t@", cnt = "@%cnt@", count = "@$ClientWaypoints.getCount());

		%o = getWord (%obj, %i);
		%o.delete();
	}
}

//function DumpAllWaypoints()
//{
//	%cnt = $ClientWaypoints.getCount();
//
//	for (%i = 0; %i < %cnt; %i++)
//	{
//		%o = $ClientWaypoints.getObject(%i);
//		%o.dump();
//		error ("Deleting waypoint "@%i@", named "@%o.getName());
//
//		if (%o.myMagic == SonicMark.WaypointMagic)
//		{
//			echo ("Magic number matches");
//			//%o.delete();
//		}
//		else
//		{
//			echo ("Magic does not match");
//		}
//	}
//}

////////////////////////////////////////////////////////////////////////
// SonicMark Class Functions
////////////////////////////////////////////////////////////////////////

function SonicMark::savePrefs (%this)
{
	error ("Saving prefs/SonicMarkPrefs.cs ...");
	export ("$SonicMark::*", "prefs/SonicMarkPrefs.cs", false);
}


// Build a list of players who are cappers, snipers, shocklancers, or teamkillers
function SonicMark::buildPlayerList (%this)
{
	%this.Player = "";
	%this.PlayerCount = 0;

	for (%i=0; true; %i++)
	{
		%p = PlayerList.findByIndex (%i);
		if (%p == $PLAYER_ERROR)
			break;

		// Make sure I don't get included in stats
		if (%p.clientId == PlayerList.getMyId())
			continue;

		// Check for grabs and caps
		%capper = StatSupport.getClientGrabs (%p.clientId) +
			  StatSupport.getClientCaps (%p.clientId);

		// Check for tks
		%tker = StatSupport.getClientTeamKills (%p.clientId);

		// Check for snipes and headshots
		%sniper = StatSupport.getClientHeadShot (%p.clientId) +
			  StatSupport.getClientKillByType (%p.clientId, "MyLaserKill");

		// Check for rearshots and shocklance kills
		%sler = StatSupport.getClientKillByType (%p.clientId, "MyRearshotKill") +
			StatSupport.getClientKillByType (%p.clientId, "MyShocklanceKill");

		// Check for kills.
		%killer = StatSupport.getClientKills (%p.clientId);

		// Check for flag returns and carrier kills.
		%fckiller = StatSupport.getClientReturns (%p.clientId) +
			    StatSupport.getClientKillByType (%p.clientId, "MyFlagCarrierKill");

		// If any of these are non zero then add the player.
		if ((%capper > 0) || (%tks > 0) || (%sniper > 0) ||
		    (%sler > 0) || (%killer > 0) || (%fckiller))
		{
			// This is a score attempting to classify the player
			%this.Player[%this.PlayerCount, "player"] = %p;
			%this.Player[%this.PlayerCount, "capper"] = %capper;
			%this.Player[%this.PlayerCount, "teamkiller"] = %tker;
			%this.Player[%this.PlayerCount, "sniper"] = %sniper;
			%this.Player[%this.PlayerCount, "shocklancer"] = %sler;
			%this.Player[%this.PlayerCount, "killer"] = %killer;
			%this.Player[%this.PlayerCount, "fckiller"] = %fckiller;
			%this.Player[%this.PlayerCount, "waypointed"] = false;
			%this.PlayerCount++;
		}
	}

	//%this.dumpPlayerInfo();
}

function SonicMark::dumpPlayerInfo (%this)
{
	echo ("***************************************************************");
	error ("SM:  Dumping player info ..................");
	echo (" ");

	for (%i=0; %i < %this.PlayerCount; %i++)
	{
		%p = %this.Player[%i, "player"];
		echo ("Player Name:        "@PlayerList.getNameByID (%p.clientId));
		echo ("       Capper:      "@%this.Player[%i, "capper"]);
		echo ("       Teamkiller:  "@%this.Player[%i, "teamkiller"]);
		echo ("       Sniper:      "@%this.Player[%i, "sniper"]);
		echo ("       Shocklancer: "@%this.Player[%i, "shocklancer"]);
		echo ("       Killer:      "@%this.Player[%i, "killer"]);
		echo ("       FC Killer:   "@%this.Player[%i, "fckiller"]);
		echo ("       Waypointed:  "@%this.Player[%i, "waypointed"]);
		echo (" ");
	}

	echo ("***************************************************************");
}

function SonicMark::PlayerIsFriendly (%this, %player)
{
	return (%player.teamId == %this.myTeam) ? true : false;
}

function SonicMark::PlayerIsEnemy (%this, %player)
{
	return (%player.teamId == %this.enemyTeam) ? true : false;
}

function SonicMark::PlayerHasFlag (%this, %player)
{
	// Check both flags to see if he has either one
	if ((FlagTracker.team[%this.myTeam].stateCurrent $= "Taken") &&
	    (FlagTracker.team[%this.myTeam].actorCurrent.clientId == %player.clientId))
		return true;

	if ((FlagTracker.team[%this.enemyTeam].stateCurrent $= "Taken") &&
	    (FlagTracker.team[%this.enemyTeam].actorCurrent.clientId == %player.clientId))
		return true;

	return false;
}

////////////////////////

// Waypoints enemy players
function SonicMark::waypointEnemies (%this, %index)
{
	// Opt for an early out
	if ((%this.PlayerCount == 0) ||
	    ($SonicMark::Enemy[%index, "max"] <= 0))
		return;

	for (%i = 0; %i < $SonicMark::Enemy[%index, "max"]; %i++)
	{
		%max = -1;
		%maxIdx = -1;

		for (%j=0; %j < %this.PlayerCount; %j++)
		{
			%p = %this.Player[%j, "player"];

			// Do not waypoint flag carrier or a teammate
			if (%this.PlayerHasFlag (%p) || %this.PlayerIsFriendly (%p))
				continue;

			// Do not included players that are already wp'd
			if ((%this.Player[%j, "waypointed"] == false) &&
			    (%this.Player[%j, %index] > %max))
			{
				%max = %this.Player[%j, %index];
				%maxIdx = %j;
			}
		}

		// See if we found one
		if (%max > 0 && %maxId >= 0)
		{
			%p = %this.Player[%maxIdx, "player"];
			AddWaypointtoClient ($SonicMark::Enemy[%index, "label"],
					     %p.clientId);
			%this.Player[%maxIdx, "waypointed"] = true;
		}
	}
}

// Waypoints firendly players
function SonicMark::waypointFriendlies (%this, %index)
{
	// Opt for an early out
	if ((%this.PlayerCount == 0) ||
	    ($SonicMark::Friendly[%index, "max"] <= 0))
		return;

	// Waypoint friendly teamkillers
	for (%i = 0; %i < $SonicMark::Friendly[%index, "max"]; %i++)
	{
		%max = -1;
		%maxIdx = -1;

		for (%j=0; %j < %this.PlayerCount; %j++)
		{
			%p = %this.Player[%j, "player"];

			// Do not waypoint flag carrier or a teammate
			if (%this.PlayerHasFlag (%p) || %this.PlayerIsEnemy (%p))
				continue;

			// Do not include players that are already wp'd
			if ((%this.Player[%j, "waypointed"] == false) &&
			    (%this.Player[%j, %index] > %max))
			{
				%max = %this.Player[%j, %index];
				%maxIdx = %j;
			}
		}

		// See if we found one
		if (%max > 0 && %maxId >= 0)
		{
			%p = %this.Player[%maxIdx, "player"];
			AddWaypointtoClient ($SonicMark::Friendly[%index, "label"],
					     %p.clientId);
			%this.Player[%maxIdx, "waypointed"] = true;
		}
	}
}

// Waypoints players in order specified by EnemyOrder and FriendlyOrder.
function SonicMark::waypointPlayers (%this)
{
	// Waypoint enemies in order
	%cnt = getWordCount ($SonicMark::EnemyOrder);
	for (%i = 0; %i < %cnt; %i++)
	{
		%index = getWord ($SonicMark::EnemyOrder, %i);
		%this.waypointEnemies (%index);

	}

	// Waypoint friendlies in order
	%cnt = getWordCount ($SonicMark::FriendlyOrder);
	for (%i = 0; %i < %cnt; %i++)
	{
		%index = getWord ($SonicMark::FriendlyOrder, %i);
		%this.waypointFriendlies (%index);

	}
}

// Refreshes waypoints to players by first clearing any waypoints and
// then redisplaying them.
function SonicMark::refreshWaypoints (%this)
{
	// Clear waypoints
	ClearWaypointsByMagic();

	if ($SonicMark::Enabled == false)
		return;

	// Build and waypoint players
	%this.buildPlayerList();
	%this.waypointPlayers();
}



// Toggle SonicMark on and off
function SM_Toggle (%val)
{
	if (%val)
	{
		$SonicMark::Enabled = !$SonicMark::Enabled;
		
		if ($SonicMark::Enabled == true)
		{
			clientCmdBottomPrint ("SonicMark is now ENABLED!", 2, 1);
			//SonicMark.refreshWaypoints();
			SM_RefreshWaypoints();
		}
		else
		{
			clientCmdBottomPrint ("SonicMark is now DISABLED!", 2, 1);
			cancel (SonicMark.Timer);
			ClearWaypointsByMagic();
		}
	}
}

// Refreshes waypoints and re-calls timer.
function SM_RefreshWaypoints()
{
	SonicMark.refreshWaypoints();
	SonicMark.Timer = schedule (SonicMark.UpdateInterval, 0, SM_RefreshWaypoints);
}

// Handle all kills.  So we can record extra info not stored StatSupport.
function SonicMark::killHandler (%this, %type, %killer, %victim, %weapon, %i_die, %i_win, %suicide, %tk)
{
	// This detects a flag carrier kill and records a stat for the killer.
	// This presumes that the kill message comes before the flag message.
	if (((FlagTracker.team[%this.enemyTeam].stateCurrent $= "Taken") &&
	     (FlagTracker.team[%this.enemyTeam].actorCurrent.clientId == %victim.clientId)) ||
	    ((FlagTracker.team[%this.myTeam].stateCurrent $= "Taken") &&
	     (FlagTracker.team[%this.myTeam].actorCurrent.clientId == %victim.clientId)))
	{
		// This message is of my own creation, anything will do
		StatMissionData.KillInfo (
			PlayerList.getNameById (%victim.clientId),
			PlayerList.getNameById (%killer.clientId),
			"MyFlagCarrierKill");
	}
	else if (%type $= "MsgLegitKill")
	{
		if (%weapon $= "laser")
		{
			StatMissionData.KillInfo (
				PlayerList.getNameById (%victim.clientId),
				PlayerList.getNameById (%killer.clientId),
				"MyLaserKill");
		}
		else if (%weapon $= "shocklance")
		{
			StatMissionData.KillInfo (
				PlayerList.getNameById (%victim.clientId),
				PlayerList.getNameById (%killer.clientId),
				"MyShocklanceKill");
		}
	}
	else if (%type $= "MsgRearshotKill")
	{
		// There are no stats collected on rearshots as of 11.29.03 in
		// Writer's Support, so this is an attempt to hack it in.
		StatMissionData.KillInfo (
			PlayerList.getNameById (%victim.clientId),
			PlayerList.getNameById (%killer.clientId),
			"MyRearshotKill");
	}
}

// Handle client drops.  Cancel refresh loop
function SonicMark::clientDropHandler (%this, %clientName, %clientID)
{
	if (%clientID == PlayerList.getMyID())
	{
		// Cancel refresh loop
		cancel (%this.Timer);
	}
}

// Handle my team changes.  So I can record my team id and the enemies.
function SonicMark::teamChangeHandler (%this, %teamId)
{
	// 0 is for observer mode
	if (%teamId != 0)
	{
		// Only valid for 2 team based games
		%this.myTeam = %teamId;
		%this.enemyTeam = (%teamId == 1) ? 2 : 1;
	}
}

package SonicMark
{
	function OptionsDlg::onWake (%this)
	{
		if (!SonicMark.KeybindsAdded)
		{
			$RemapName[$RemapCount] = "SonicMark Toggle";
			$RemapCmd[$RemapCount] = "SM_Toggle";
			$RemapCount++;
			SonicMark.KeybindsAdded = true;
		}

		parent::onWake (%this);
	}

	// Delay adding callbacks to prevent warnings
	function LoadingGui::onWake (%this)
	{
		parent::onWake (%this);

		Callback.add (MyTeamChanged, "SonicMark.teamChangeHandler");
		Callback.add (KillCallback, "SonicMark.killHandler");
		Callback.add (onUserClientDrop, "SonicMark.clientDropHandler");

		if ($SonicMark::Enabled == true)
		{
			echo ("SM: Starting update timer ....");
			SM_RefreshWaypoints();
		}
	}

	function quit()
	{
		export ("$SonicMark::*", "prefs/SonicMarkPrefs.cs", false);
		parent::quit();
	}
};

activatePackage ("SonicMark");

//if (!isObject (SonicMarkOptions))
exec ("gui/SonicMarkOptions.gui");

//////////////////////////////////////////////////////////////
// SonicMark Options Dialog
//

// Return index of the word in the list else -1
function getWordIndex (%list, %word)
{
	for (%i=0; %i < getWordCount (%list); %i++)
	{
		if (getWord (%list, %i) $= %word)
			return %i;
	}
	return -1;
}

// Add the word to the list, do nothing it already exists
function addWordToList (%list, %word)
{
	%index = getWordIndex (%list, %word);
	if (%index == -1)
		return (%list $= "") ? %word : %list SPC %word;
	return %list;
}

// Remove the word from the list if it exists
function removeWordInLIst (%list, %word)
{
	// Nasty hack to make removeWord() behave right
	for (%i=0; %i < getWordCount (%list); %i++)
	{
		%tmp = getWord (%list, %i);
		if (%tmp !$= %word)
			%newList = (%newList $= "") ? %tmp : %newList SPC %tmp;
	}
	return %newList;
}

function SonicMarkOptions::onWake (%this)
{
	// Set the enemy tab as the default
	%this.setPane ("Enemy");

	// Initialize the states of all the toggles
	%this.onEnemyToggle (capper);
	%this.onEnemyToggle (sniper);
	%this.onEnemyToggle (shocklancer);
	%this.onEnemyToggle (killer);
	%this.onEnemyToggle (fckiller);
	%this.onFriendlyToggle (capper);
	%this.onFriendlyToggle (teamkiller);

	// Set num sliders
	SonicMark_EnemyCapper_Num_Slider.setValue ($SonicMark::Enemy["capper", "max"]);
	SonicMark_EnemySniper_Num_Slider.setValue ($SonicMark::Enemy["sniper", "max"]);
	SonicMark_EnemyShocklancer_Num_Slider.setValue ($SonicMark::Enemy["shocklancer", "max"]);
	SonicMark_EnemyKiller_Num_Slider.setValue ($SonicMark::Enemy["killer", "max"]);
	SonicMark_EnemyFCKiller_Num_Slider.setValue ($SonicMark::Enemy["fckiller", "max"]);

	SonicMark_FriendlyCapper_Num_Slider.setValue ($SonicMark::Friendly["capper", "max"]);
	SonicMark_FriendlyTeamkiller_Num_Slider.setValue ($SonicMark::Friendly["teamkiller", "max"]);

	SonicMarkOptions.updateEnemySliders();
	SonicMarkOptions.updateFriendlySliders();
}

// Set pane function
function SonicMarkOptions::setPane (%this, %pane)
{
	if (%pane $= "")
		return;

	if (%this.pane $= "")
		%this.pane = %pane;

	// %this.pane is the current tab
	%curPane = "SonicMark_" @ %this.pane @ "Pane";
	%curTab = "SonicMark_" @ %this.pane @ "Tab";
	%curPane.setVisible (false);
	%curTab.setValue (false);

	// %pane is the new
	%newPane = "SonicMark_" @ %pane @ "Pane";
	%newTab = "SonicMark_" @ %pane @ "Tab";
	%newPane.setVisible (true);
	%newTab.setValue (true);
	%this.pane = %pane;
}

function SonicMarkOptions::onEnemyToggle (%this, %type)
{
	// Set the visibility of the type
	%toggle = "SonicMark_Enemy" @ %type @ "_Toggle";
	%field = "SonicMark_Enemy" @ %type @ "_Field";
	%field.setVisible (%toggle.getValue());
	%order = $SonicMark::EnemyOrder;

	// If toggle is true then add word to list, else remove it
	if (%toggle.getValue ())
		%order = addWordToList (%order, %type);
	else
		%order = removeWordInList (%order, %type);

	$SonicMark::EnemyOrder = %order;
}

function SonicMarkOptions::onFriendlyToggle (%this, %type)
{
	// Set the visibility of the type
	%toggle = "SonicMark_Friendly" @ %type @ "_Toggle";
	%field = "SonicMark_Friendly" @ %type @ "_Field";
	%field.setVisible (%toggle.getValue());
	%order = $SonicMark::FriendlyOrder;

	// If toggle is true then add word to list, else remove it
	if (%toggle.getValue ())
		%order = addWordToList (%order, %type);
	else
		%order = removeWordInList (%order, %type);

	$SonicMark::FriendlyOrder = %order;
}

// Updates sliders and values based on EnemyOrder
function SonicMarkOptions::updateEnemySliders (%this)
{
	for (%i=0; %i < getWordCount ($SonicMark::EnemyOrder); %i++)
	{
		switch$ (getWord ($SonicMark::EnemyOrder, %i))
		{
		case "capper":
			SonicMark_EnemyCapper_Priority_Value.setValue (%i);
			SonicMark_EnemyCapper_Priority_Slider.setValueActive = true;
			SonicMark_EnemyCapper_Priority_Slider.setValue (%i);
			SonicMark_EnemyCapper_Priority_Slider.setValueActive = false;
		case "sniper":
			SonicMark_EnemySniper_Priority_Value.setValue (%i);
			SonicMark_EnemySniper_Priority_Slider.setValueActive = true;
			SonicMark_EnemySniper_Priority_Slider.setValue (%i);
			SonicMark_EnemySniper_Priority_Slider.setValueActive = false;
		case "shocklancer":
			SonicMark_EnemyShocklancer_Priority_Value.setValue (%i);
			SonicMark_EnemyShocklancer_Priority_Slider.setValueActive = true;
			SonicMark_EnemyShocklancer_Priority_Slider.setValue (%i);
			SonicMark_EnemyShocklancer_Priority_Slider.setValueActive = false;
		case "killer":
			SonicMark_EnemyKiller_Priority_Value.setValue (%i);
			SonicMark_EnemyKiller_Priority_Slider.setValueActive = true;
			SonicMark_EnemyKiller_Priority_Slider.setValue (%i);
			SonicMark_EnemyKiller_Priority_Slider.setValueActive = false;
		case "fckiller":
			SonicMark_EnemyFCKiller_Priority_Value.setValue (%i);
			SonicMark_EnemyFCKiller_Priority_Slider.setValueActive = true;
			SonicMark_EnemyFCKiller_Priority_Slider.setValue (%i);
			SonicMark_EnemyFCKiller_Priority_Slider.setValueActive = false;
		}
	}
}

// Update sliders and values based on FriendlyOrder
function SonicMarkOptions::updateFriendlySliders (%this)
{
	for (%i=0; %i < getWordCount ($SonicMark::FriendlyOrder); %i++)
	{
		switch$ (getWord ($SonicMark::FriendlyOrder, %i))
		{
		case "capper":
			SonicMark_FriendlyCapper_Priority_Value.setValue (%i);
			SonicMark_FriendlyCapper_Priority_Slider.setValueActive = true;
			SonicMark_FriendlyCapper_Priority_Slider.setValue (%i);
			SonicMark_FriendlyCapper_Priority_Slider.setValueActive = false;
		case "teamkiller":
			SonicMark_FriendlyTeamkiller_Priority_Value.setValue (%i);
			SonicMark_FriendlyTeamkiller_Priority_Slider.setValueActive = true;
			SonicMark_FriendlyTeamkiller_Priority_Slider.setValue (%i);
			SonicMark_FriendlyTeamkiller_Priority_Slider.setValueActive = false;
		}
	}
}

// Tries to create new order by shuffling around values
function SonicMarkOptions::setEnemyPriority (%this, %type, %index)
{
	%cnt = 5;
	%order = "";
	%old = -1;

	// Build an array from the list including "" or disabled types
	for (%i=0; %i < %cnt; %i++)
	{
		%word = getWord ($SonicMark::EnemyOrder, %i);
		if (%word $= %type)
			%old = %i;

		%order[%i] = %word;
	}

	// Nothing to do
	if ((%old < 0) || (%old == %index))
		return $SonicMark::EnemyOrder;

	// Now swap the values
	%tmp = %order[%index];
	%order[%index] = %type;
	%order[%old] = %tmp;

	// Rebuilt the list after the shuffling
	for (%i=0; %i < %cnt; %i++)
	{
		if (%order[%i] $= "")
			continue;

		%newOrder = (%i == 0) ? %order[%i] : %newOrder SPC %order[%i];
	}

	return %newOrder;
}

function SonicMarkOptions::setFriendlyPriority (%this, %type, %index)
{
	%cnt = 2;
	%order = "";
	%old = -1;

	// Build an array from the list including "" or disabled types
	for (%i=0; %i < %cnt; %i++)
	{
		%word = getWord ($SonicMark::FriendlyOrder, %i);
		if (%word $= %type)
			%old = %i;

		%order[%i] = %word;
	}

	// Nothing to do
	if ((%old < 0) || (%old == %index))
		return $SonicMark::FriendlyOrder;

	// Now swap the values
	%tmp = %order[%index];
	%order[%index] = %type;
	%order[%old] = %tmp;

	// Rebuilt the list after the shuffling
	for (%i=0; %i < %cnt; %i++)
	{
		if (%order[%i] $= "")
			continue;

		%newOrder = (%i == 0) ? %order[%i] : %newOrder SPC %order[%i];
	}

	return %newOrder;
}

function SonicMark_EnemyCapper_Priority_Slider::onSlide (%this, %value)
{
	// If we are being called as the result of a setValue then return
	if (%this.setValueActive)
		return;

	$SonicMark::EnemyOrder = SonicMarkOptions.setEnemyPriority ("capper", %value);
	SonicMarkOptions.updateEnemySliders();
}

function SonicMark_EnemySniper_Priority_Slider::onSlide (%this, %value)
{
	// If we are being called as the result of a setValue then return
	if (%this.setValueActive)
		return;

	$SonicMark::EnemyOrder = SonicMarkOptions.setEnemyPriority ("sniper", %value);
	SonicMarkOptions.updateEnemySliders();
}

function SonicMark_EnemyShocklancer_Priority_Slider::onSlide (%this, %value)
{
	// If we are being called as the result of a setValue then return
	if (%this.setValueActive)
		return;

	$SonicMark::EnemyOrder = SonicMarkOptions.setEnemyPriority ("shocklancer", %value);
	SonicMarkOptions.updateEnemySliders();
}

function SonicMark_EnemyKiller_Priority_Slider::onSlide (%this, %value)
{
	// If we are being called as the result of a setValue then return
	if (%this.setValueActive)
		return;

	$SonicMark::EnemyOrder = SonicMarkOptions.setEnemyPriority ("killer", %value);
	SonicMarkOptions.updateEnemySliders();
}

function SonicMark_EnemyFCKiller_Priority_Slider::onSlide (%this, %value)
{
	// If we are being called as the result of a setValue then return
	if (%this.setValueActive)
		return;

	$SonicMark::EnemyOrder = SonicMarkOptions.setEnemyPriority ("fckiller", %value);
	SonicMarkOptions.updateEnemySliders();
}

function SonicMark_FriendlyCapper_Priority_Slider::onSlide (%this, %value)
{
	// If we are being called as the result of a setValue then return
	if (%this.setValueActive)
		return;

	$SonicMark::FriendlyOrder = SonicMarkOptions.setFriendlyPriority ("capper", %value);
	SonicMarkOptions.updateFriendlySliders();
}

function SonicMark_FriendlyTeamkiller_Priority_Slider::onSlide (%this, %value)
{
	// If we are being called as the result of a setValue then return
	if (%this.setValueActive)
		return;

	$SonicMark::FriendlyOrder = SonicMarkOptions.setFriendlyPriority ("teamkiller", %value);
	SonicMarkOptions.updateFriendlySliders();
}


